#%!/usr/bin/env python
"""
    Module: mcd.py

    Package Dependency: None
    Description: 
        This is a standalone MCA(Machine Check Architecture) decoder.
        Users could run the script as 1) manual decoding; or 2) pass in command line parameters.
        Please see the usage instruction document in the same script folder.

    Autor: Jerr Chen
    Changed: Yaco Xu
    Date: August 14, 2014
"""

import textwrap as _textwrap
import sys
#key less than 255 mean bit offset
#key more than 255 mean, this is a bit range not a bit offset, high byte is high bit offset, low byte is low bit offset
csrList={
"IIO_ERRPINSTS":{
    'desc': 'This register holds the error pin status.',
    2:  'ERR2 - State of ERROR[2] pin (the register value is active high, the socket pin is active VSS)',
    1:  'ERR1 - State of ERROR[1] pin (the register value is active high, the socket pin is active VSS)',
    0:  'ERR0 - State of ERROR[0] pin (the register value is active high, the socket pin is active VSS)'
},
"IIO_GC_ERRST":{
    'desc': 'This register indicates the correctable error reported to the IIO global error logic. An individual error status bit that is set indicates that local IIO stack interface has detected an error.',
    0x1f00:   'bitmap -- Bit map status of correctable error\n(00:00) "Stack0/C-Stack"\n(01:01) "Stack1/P-Stack"\n(02:02) "Stack2/P-Stack"\n(03:03) "Stack3/P-Stack"\n(04:04) "Stack4/P-Stack"\n(05:05) "VPP errors"'
    },
"IIO_GF_ERRST":{
    'desc': 'This register indicates the fatal error reported to the IIO global error logic. An individual error status bit that is set indicates that local IIO stack interface has detected an error.',
    'sameTo':'IIO_GC_ERRST'
    },
"IIO_GNF_ERRST":{
    'desc': 'This register indicates the non-fatal error reported to the IIO global error logic. An individual error status bit that is set indicates that local IIO stack interface has detected an error.',
    'sameTo':'IIO_GC_ERRST'
    },
"IIO_GSYSST":{
    'desc': 'This register indicates the error severity signaled by the IIO global error logic. Setting of an individual error status bit indicates that the corresponding error severity has been detected by the IIO.',
    2:   'SEVERITY2 - When set, IIO has detected an error of error severity 2',
    1:   'SEVERITY1 - When set, IIO has detected an error of error severity 1',
    0:   'SEVERITY0 - When set, IIO has detected an error of error severity 0'
    },
#"IIO_GSYSMAP":{
#    'desc': 'This register maps the error severity detected by the IIO to on of the system events. When an error is detected by the IIO, its corresponding error severity #determines which system event to generate according to this register.',
#    0x0806:  'SEVERITY2_MAP - 010: Generate U2C NMI event to local cores only\n001: Generate CSMI VLW\n000: No inband message\nOthers: Reserved',
#    0x0503:  'SEVERITY1_MAP - 010: Generate U2C NMI event to local cores only\n001: Generate CSMI VLW\n000: No inband message\nOthers: Reserved',
#    0x0200:  'SEVERITY0_MAP - 010: Generate U2C NMI event to local cores only\n001: Generate CSMI VLW\n000: No inband message\nOthers: Reserved'
#    },
'UBOXErrSts':{
    'desc': 'This is error status register in the Ubox and covers most of the interrupt related errors.',
    0x1712:  'Msg_Ch_Tkr_TimeOut - Message Channel Tracker TimeOut. This error occurs when any NP request doesn\'t receive response in 4K cycles.',
    17: 'Msg_Ch_Tkr_Err - Message Channel Tracker Error. This error occurs such case that illegal broad cast port ID access to the message channel. ',
    16:  'SMI_delivery_valid - SMI interrupt delivery status valid, write 1\'b0 to clear valid status.',
    7: 'MasterLockTimeOut - Master Lock Timeout received by Ubox.',
    6: 'SMITimeOut - SMI Timeout received by Ubox.',
    5: 'CFGWrAddrMisAligned - MMCFG Write Address Misalignment received by Ubox.',
    4: 'CFGRdAddrMisAligned - MMCFG Read Address Misalignment received by Ubox.',
    3: 'UnsupportedOpcode - Unsupported opcode received by Ubox',
    2: 'PoisonRsvd - Ubox received a poisoned transaction.',
    1: 'SMISrciMC - SMI is caused due to an imdication from the IMC.',
    0: 'SMISrcUMC - This is a bit that indicates that an SMI was caused due to a locally generated UMC.',
    },
'IerrLoggingReg':{
    'desc': 'IERR first/second logging error.',
    25:  'SecondIerrSrcFromCbo - Set to 1 if the second SrcID is from a Cbo or core.',
    24:  'SecondIerrSrcValid - set to 1 if the SecondIerrSrcID is valid.',
    0x1710:  'SecondIerrSrcId - If SecondIerrSrcValid is 1, the block responsible for generating the second IERR is logged here. Refer to decode table in FirstIerrSrcID.',
    9:  'FirstIerrSrcFromCbo - Set to 1 of the FirstIerrSrcID is from a Cbo or core.',
    8:  'FirstIerrSrcValid - Set to 1 if the FirstSrcID is valid.',
    0x0700: 'FirstIerrSrcId - msgCh portID of the end point with the first IERR. If FirstIerrSrcValid is 1, the block responsible for generating the first IERR is decoded as follows:\n01000100 - PCU\n{2\'b11,6\'bnnnnnn} - CHAID == 6\'bnnnnnn'
    },
"IERRLOGINGREG":{
    'desc': 'IERR first/second logging error.',
    'sameTo': 'IerrLoggingReg'
    },
'MCerrLoggingReg':{
    'desc': 'MCERR first/second logging error.',
    0x1d1a: 'SecondMCerrSrcSubPortId - msgCh Sub PortID of the EP with the first MCERR',
    25:  'SecondMCerrSrcFromCbo - Set to 1 if the second SrcID is from a Cbo or CORE. ',
    24:  'SecondMCerrSrcValid - Set to 1 if the SecondMCerrSrcID is valid.',
    0x1710:  'SecondMCerrSrcId - If SecondMCerrSrcValid is 1, the block responsible for generating the second MCERR is logged here. Refer to decode table in FirstMCerrSrcID.',
    0x0d0a: 'FirstMCerrSrcSubPortId --msgCh Sub PortID of the EP with the first MCERR. The subPortID here is important for MC to specify a channel, but not applied to other clusters.\n0001 - MC MAIN\n0010 - MC 2LM0\n0011 - MC 2LM1\n0100 - MC 2LM2\n0101 - MC LMSCH0\n0110 - MC LMDP0\n0111 - MC LMSCH1\n1000 - MC LMDP1\n1001 - MC LMSCH2\n1010 - MC LMDP2',
    9:  'FirstMCerrSrcFromCbo - Set to 1 of the FirstMCerrSrcID is from a Cbo or CORE.',
    8:  'FirstMCerrSrcValid - Set to 1 if the FirstSrcID is valid.',
    0x0700: 'FirstMCerrSrcId - msgCh portID of the EP with the first MCERR. If FirstMCerrSrcValid is 1, the block responsible for generating the first MCERR is decoded as follows:\n00000010 - Intel UPI 0\n00000011 - Intel UPI 1\n00000110 - Intel UPI 2\n01000100 - PCU\n01100000 - IMC 0, channel by subPortID\n01100100 - IMC 1, channel by subPortID\n01100010 - M2MEM0\n01100110 - M2MEM1\n101000xx - IIO0\n101001xx - IIO1\n101010xx - IIO2\n101011xx - CBDMA\n101100xx - MCP0\n101101xx - MCP1\n{2\'b11,6\'bnnnnnn} - CHAID == 6\'bnnnnnn',
    },
'MCERRLOGGINGREG':{
    'desc': 'MCERR first/second logging error.',
    'sameTo': 'MCerrLoggingReg'
    },
'EMCA_CORE_CSMI_LOG':{
    'desc': 'This is a log of which cores have signalled a CSMI to Ubox via the MCLK message.',
    27:  'Core27 - CSMI received indicator for core 27',
    26:  'Core26 - CSMI received indicator for core 26',
    25:  'Core25 - CSMI received indicator for core 25',
    24:  'Core24 - CSMI received indicator for core 24',
    23:  'Core23 - CSMI received indicator for core 23',
    22:  'Core22 - CSMI received indicator for core 22',
    21:  'Core21 - CSMI received indicator for core 21',
    20:  'Core20 - CSMI received indicator for core 20',
    19:  'Core19 - CSMI received indicator for core 19',
    18:  'Core18 - CSMI received indicator for core 18',
    17:  'Core17 - CSMI received indicator for core 17',
    16:  'Core16 - CSMI received indicator for core 16',
    15:  'Core15 - CSMI received indicator for core 15',
    14:  'Core14 - CSMI received indicator for core 14',
    13:  'Core13 - CSMI received indicator for core 13',
    12:  'Core12 - CSMI received indicator for core 12',
    11:  'Core11 - CSMI received indicator for core 11',
    10:  'Core10 - CSMI received indicator for core 10',
    9:  'Core9 - CSMI received indicator for core 9',
    8:  'Core8 - CSMI received indicator for core 8',
    7:  'Core7 - CSMI received indicator for core 7',
    6:  'Core6 - CSMI received indicator for core 6',
    5:  'Core5 - CSMI received indicator for core 5',
    4:  'Core4 - CSMI received indicator for core 4',
    3:  'Core3 - CSMI received indicator for core 3',
    2:  'Core2 - CSMI received indicator for core 2',
    1:  'Core1 - CSMI received indicator for core 1',
    0:  'Core0 - CSMI received indicator for core 0'
    },
'EMCA_CORE_CSMI':{
    'desc': 'This is a log of which cores have signalled a CSMI to Ubox via the MCLK message.',
    'sameTo': 'EMCA_CORE_CSMI_LOG'
    },
'EMCA_CORE_MSMI_LOG':{
    'desc': 'This is a log of which cores have signalled an MSMI to ubox via the MCLK message.',
    27:  'Core27 - MSMI received indicator for core 27',
    26:  'Core26 - MSMI received indicator for core 26',
    25:  'Core25 - MSMI received indicator for core 25',
    24:  'Core24 - MSMI received indicator for core 24',
    23:  'Core23 - MSMI received indicator for core 23',
    22:  'Core22 - MSMI received indicator for core 22',
    21:  'Core21 - MSMI received indicator for core 21',
    20:  'Core20 - MSMI received indicator for core 20',
    19:  'Core19 - MSMI received indicator for core 19',
    18:  'Core18 - MSMI received indicator for core 18',
    17:  'Core17 - MSMI received indicator for core 17',
    16:  'Core16 - MSMI received indicator for core 16',
    15:  'Core15 - MSMI received indicator for core 15',
    14:  'Core14 - MSMI received indicator for core 14',
    13:  'Core13 - MSMI received indicator for core 13',
    12:  'Core12 - MSMI received indicator for core 12',
    11:  'Core11 - MSMI received indicator for core 11',
    10:  'Core10 - MSMI received indicator for core 10',
    9:  'Core9 - MSMI received indicator for core 9',
    8:  'Core8 - MSMI received indicator for core 8',
    7:  'Core7 - MSMI received indicator for core 7',
    6:  'Core6 - MSMI received indicator for core 6',
    5:  'Core5 - MSMI received indicator for core 5',
    4:  'Core4 - MSMI received indicator for core 4',
    3:  'Core3 - MSMI received indicator for core 3',
    2:  'Core2 - MSMI received indicator for core 2',
    1:  'Core1 - MSMI received indicator for core 1',
    0:  'Core0 - MSMI received indicator for core 0'
    },
'EMCA_CORE_MSMI':{
    'desc': 'This is a log of which cores have signalled an MSMI to ubox via the MCLK message.',
    'sameTo': 'EMCA_CORE_MSMI_LOG'
    },
'BIST_RESULTS':{
    'desc': 'Config read-only access to core BIST results',
    0x1b00:  'results - 0 - BIST failure, 1 - BIST pass'
    },
'CORE_FIVR_ERR_LOG':{
    'desc': 'Reports Core FIVR Faults',
    0x1f00:  'FAULT_VECTOR - Fault vector - Bits correspond to cores which have a core IVR fault. Cores with a core IVR fault will not come out of reset. This field has the same field mapping as the CSR RESOLVED_CORES and results are reported in CSR RESOLVED_CORES.'
    },
'UNCORE_FIVR_ERR_LOG':{
    'desc': 'Reports Uncore FIVR Faults',
    0x1f00:  'FAULT_VECTOR - If any bit are set in 31:0 an uncore FIVR fault has occurred. Firmware can attempt to FRB the socket. An IERR will also occur on this condition.'
    },
'correrrcnt_0':{
    'desc': 'Per Rank corrected error counters.',
    'defFunReg': 'CORRERRCNT DEV_FUN_REG(0x0d,0x07,0x104)|CORRERRCNT DEV_FUN_REG(0x0d,0x03,0x104)|CORRERRCNT DEV_FUN_REG(0x0c,0x07,0x104)|CORRERRCNT DEV_FUN_REG(0x0c,0x03,0x104)|CORRERRCNT DEV_FUN_REG(0x0b,0x07,0x104)|CORRERRCNT DEV_FUN_REG(0x0b,0x03,0x104)|CORRERRCNT DEV_FUN_REG(0x0a,0x07,0x104)|CORRERRCNT DEV_FUN_REG(0x0a,0x03,0x104)',
    31:  'overflow_1 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0x1e10: 'cor_err_cnt_1 - The corrected error count for this rank.',
    15:  'overflow_0 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0xe00: 'cor_err_cnt_0 - The corrected error count for this rank.',
    },
'correrrcnt_1':{
    'desc': 'Per Rank corrected error counters.',
    'defFunReg': 'CORRERRCNT DEV_FUN_REG(0x0d,0x07,0x108)|CORRERRCNT DEV_FUN_REG(0x0d,0x03,0x108)|CORRERRCNT DEV_FUN_REG(0x0c,0x07,0x108)|CORRERRCNT DEV_FUN_REG(0x0c,0x03,0x108)|CORRERRCNT DEV_FUN_REG(0x0b,0x07,0x108)|CORRERRCNT DEV_FUN_REG(0x0b,0x03,0x108)|CORRERRCNT DEV_FUN_REG(0x0a,0x07,0x108)|CORRERRCNT DEV_FUN_REG(0x0a,0x03,0x108)',
    31:  'overflow_3 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0x1e10: 'cor_err_cnt_3 - The corrected error count for this rank.',
    15:  'overflow_2 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0xe00: 'cor_err_cnt_2 - The corrected error count for this rank.',
    },
'correrrcnt_2':{
    'desc': 'Per Rank corrected error counters.',
    'defFunReg': 'CORRERRCNT DEV_FUN_REG(0x0d,0x07,0x10c)|CORRERRCNT DEV_FUN_REG(0x0d,0x03,0x10c)|CORRERRCNT DEV_FUN_REG(0x0c,0x07,0x10c)|CORRERRCNT DEV_FUN_REG(0x0c,0x03,0x10c)|CORRERRCNT DEV_FUN_REG(0x0b,0x07,0x10c)|CORRERRCNT DEV_FUN_REG(0x0b,0x03,0x10c)|CORRERRCNT DEV_FUN_REG(0x0a,0x07,0x10c)|CORRERRCNT DEV_FUN_REG(0x0a,0x03,0x10c)',
    31:  'overflow_5 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0x1e10: 'cor_err_cnt_5 - The corrected error count for this rank.',
    15:  'overflow_4 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0xe00: 'cor_err_cnt_4 - The corrected error count for this rank.',
    },
'correrrcnt_3':{
    'desc': 'Per Rank corrected error counters.',
    'defFunReg': 'CORRERRCNT DEV_FUN_REG(0x0d,0x07,0x110)|CORRERRCNT DEV_FUN_REG(0x0d,0x03,0x110)|CORRERRCNT DEV_FUN_REG(0x0c,0x07,0x110)|CORRERRCNT DEV_FUN_REG(0x0c,0x03,0x110)|CORRERRCNT DEV_FUN_REG(0x0b,0x07,0x110)|CORRERRCNT DEV_FUN_REG(0x0b,0x03,0x110)|CORRERRCNT DEV_FUN_REG(0x0a,0x07,0x110)|CORRERRCNT DEV_FUN_REG(0x0a,0x03,0x110)',
    31:  'overflow_7 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0x1e10: 'cor_err_cnt_7 - The corrected error count for this rank.',
    15:  'overflow_6 - The corrected error count for this rank has been overflowed. Once set it can only be cleared via a write from BIOS.',
    0xe00: 'cor_err_cnt_6- The corrected error count for this rank.',
    },
'correrrorstatus':{
    'desc': 'Per Rank corrected error status. These bits are reset by bios.',
    0x150e:  'ddr4crc_rank_log - This field get set with 1\'b1, if the corresponding rank detected ddr4 crc in one of its write data. This will be cleared by BIOS.',
    0x0d0b:  'dimm_alert_select',
    0x0a08:  'dimm_alert',
    0x0700:  'err_overflow_stat - This 8 bit field is the per rank error over-threshold status bits. The organization is as follows:\nBit 0 : Rank 0\nBit 1 : Rank 1\nBit 2 : Rank 2\nBit 3 : Rank 3\nBit 4 : Rank 4\nBit 5 : Rank 5\nBit 6 : Rank 6\nBit 7 : Rank 7'
},
'MCA_ERR_SRC_LOG':{
    'desc': 'Thi CSR is used by the PCU to log the error sources. This register is initialized to zeroes during reset. The PCU will set the relevant bits when the condition they represent appears. The PCU never clears the registers. The UBox or platform entities should clear them when they are consumed, unless their processing involves taking down the platform.',
    31: 'CATERR - External error: The package observed CATERR# (for any reason).It is OR (bit 30, bit29); functions as a Valid bit for the other two package conditions. It has no effect when a local core is associated with the error.',
    30: 'IERR - External error: The package observed IERR.',
    29: 'MCERR - External error: The package observed MCERR.',
    28: 'CATERR_INTERNAL - Internal error: This socket asserted a CATERR#. This is OR (bit27,26).',
    27: 'IERR_INTERNAL - Internal error: This socket asserted IERR.',
    26: 'MCERR_INTERNAL - Internal error: This socket asserted MCERR.',
    23: 'MSMI - External error: The package observed MSMI# (for any reason).It is or(bit 22, bit21); functions as a Valid bit for the other two package conditions. It has no effect when a local core is associated with the error.',
    22: 'MSMI_IERR - External error: The package observed MSMI_IERR.',
    21: 'MSMI_MCERR - External error: The package observed MSMI_MCERR.',
    20: 'MSMI_INTERNAL - Internal error: This socket asserted a MSMI#. This is OR (bit19,18).',
    19: 'MSMI_IERR_INTERNAL - Internal error: This socket asserted MSMI_IERR.',
    18: 'MSMI_MCERR_INTERNAL - Internal error: This socket asserted MSMI_MCERR.',
    0x0400: 'FIRST_CORE_IERR - Reserved. SKX b302820.'
    },
'MCA_EER_SRC':{
    'desc': 'Thi CSR is used by the PCU to log the error sources. This register is initialized to zeroes during reset. The PCU will set the relevant bits when the condition they represent appears. The PCU never clears the registers. The UBox or platform entities should clear them when they are consumed, unless their processing involves taking down the platform.',
    'sameTo': 'MCA_ERR_SRC_LOG'
    },
"R2PINGERRLOG0": {
    'desc': 'R2PCIe Ingress error log 0',
        15: 'IVSnpCrdOverFlow - IV Snoop credit overflow',
        14: 'IVBgfCrdOverFlow - IV BGF credit overflow',
        13: 'UbxIioNcsCreditOverFlow',
        12: 'UbxIioNcbCreditOverFlow',
        11: 'ParErrIng0IngDone - Indicating parity error injection is done. S/W need clear this bit if it wants to inject another error.',
        10: 'ParErrIng0 - Ingress BL data parity error - Agent 0',
        9: 'IioNcsCrdOverFlow - IIO NCS credit overflow',
        8: 'IioNcbCrdOverFlow - IIO NCB credit overflow.',
        7: 'IioIdiCrdOverFlow - IIO IDI credit overflow.',
        6: 'UbxQpiNcsCrdOverFlow - Ubox Intel UPI NCS credit overflow.',
        5: 'UbxQpiNcbCrdOverFlow - Ubox Intel UPI NCB credit overflow',
        4: 'UbxCboNcsCrdOverFlow - Ubox Cbo NCS credit overflow',
        3: 'UbxCboNcbCrdOverFlow - Ubox Cbo NCB credit overflow',
        2: 'BLBgfCrdOverFlow - BL BGF credit overflow.',
        1: 'AK1BgfCrdOverFlow - AK BGF 1 BGF credit overflow.',
        0: 'AK0BgfCrdOverFlow - AK BGF 0 BGF credit overflow.'
    },
"R2GLERRCFG": {
        'desc': 'R2PCIe global viral/fatal error configuration',
        20: 'FatalStatusFromUbox - Read only from Ubox fatal status',
        19: 'ViralStatusFromUbox - Read only from Ubox viral status',
        18: 'FatalStatusFromIio - Read only from IIO fatal status',
        17: 'ViralStatusFromIio - Read only from IIO viral status',
        16: 'ViralStatusToIio - Read only to IIO viral status',
        15: 'LocalErrorStatus - Read only R2PCIe error status',
        0x0e0d: 'MaskR2FatalError - Set to 0x3 to mask R2PCIE errors from the global fatal error status. Set to 0x0 to not mask. All others reserved.',
        10: 'MaskIIOViralIn - Set 1 to block Viral status from IIO to global viral status',
        9: 'MaskIIOViralOut - Set 1 to block Viral status from global viral status to IIO',
        7: 'MaskIIOFatalErrorIn - Set 1 to block fatal error status from IIO to global fatal status',
        6: 'MaskUboxFatalErrorIn - Set 1 to block fatal error status of Ubox to global fatal status',
        5: 'MaskUboxViralIn - Set 1 to block viral status of Ubox',
        4: 'ResetGlobalViral - Set 1 to force clear global viral status.',
        3: 'ResetGlobalFatalError - Set 1 to force clear global fatal error status.'
    },
"R2EGRERRLOG": {
        'desc': 'R2PCIe Egress error log',
        29: 'ADEgress1_Overflow - AD Egress Agent 1 buffer overflow',
        28: 'ADEgress0_Overflow - AD Egress Agent 0 buffer overflow',
        27: 'BLEgress1_Overflow - BL Egress Agent 1 buffer overflow',
        26: 'BLEgress0_Overflow - BL Egress Agent 0 buffer overflow',
        25: 'AKEgress1_Overflow - AK Egress Agent 1 buffer overflow',
        24: 'AKEgress0_Overflow - AK Egress Agent 0 buffer overflow',
        23: 'ADEgress1_Write_to_Valid_Entry - AD Egress Agent 1 write to occupied entry',
        22: 'ADEgress0_Write_to_Valid_Entry - AD Egress Agent 0 write to occupied entry',
        21: 'BLEgress1_Write_to_Valid_Entry - BL Egress Agent 1 write to occupied entry',
        20: 'BLEgress0_Write_to_Valid_Entry - BL Egress Agent 0 write to occupied entry',
        19: 'AKEgress1_Write_to_Valid_Entry - AK Egress Agent 1 write to occupied entry',
        18: 'AKEgress0_Write_to_Valid_Entry - AK Egress Agent 0 write to occupied entry',
    },
}

import mcd
def perseOneCsr(regName, regValue):
    #print regName, regValue
    retValue=""            
    vec_status = mcd.BitVector(long(regValue,16))
    if vec_status[31:0] == 0:
         return ""
    if regName in csrList.keys():
        sameToKey=regName
        if "sameTo" in csrList[regName].keys():
            sameToKey=csrList[regName]["sameTo"]
        for key in csrList[sameToKey].keys():
            if  key == "desc" or key == "defFunReg":
                continue
            if key > 255:
                #print key
                lowOffset=key&0x00ff
                highOffset=(key>>8)&0x00ff
                if vec_status[highOffset:lowOffset]  != 0:
                    retValue=retValue+csrList[regName]['desc']+"\n["+str(highOffset)+":"+str(lowOffset)+"]  = "+str(vec_status[highOffset:lowOffset]) +"\n"+csrList[sameToKey][key]
                    #print highOffset, lowOffset, "to", vec_status[highOffset:lowOffset] , csrList[regName][key]
            else:
                if vec_status[key:key] != 0:
                    #print "bit:",key," is one", csrList[regName][key]
                    retValue=retValue+csrList[regName]['desc']+"\n["+str(key)+"]"+csrList[sameToKey][key]

    else:
        return ""

    return retValue

def main():
    #print csrList["gcerrst"]["desc"], csrList["gcerrst"][26]
    #print csrList["gnerrst"]["desc"], csrList["gnerrst"][0]
        print csrList["MCA_ERR_SRC_LOG"][18]
        #veco= perseOneCsr("MCA_ERR_SRC_LOG","0x00d40000")
      
      
if __name__ == "__main__":
    main()

